#include "controller.hpp"


/**
 * For most of this project I took care to simplify my code once I got 
 * it working, avoided repeating code, and tried to make all of my 
 * functions be short and easy to understand. Then I added a bunch of
 * features that I put no further work into once they were functional
 * because the deadline was approaching. Most of those features are 
 * contained in this constructor, which is very bloated and probably 
 * difficult to read. The program as a whole is also mostly lacking
 * in comments, hopefully the function names are descriptive enough.
 */
controller::controller(QWidget* window): QWidget(window){
	auto new_node_button = new QPushButton("New Node", this);
    new_node_button->setGeometry(0, 0, 200, 60);
    QPushButton::connect(new_node_button, &QPushButton::clicked, this, &controller::new_node);

    new mode_switch(this);
    selected = new selected_pane(this);


    //TODO: Create some sort of "Mass actions" class to house these
    QLabel* mass_actions = new QLabel("Mass Actions", this);
    mass_actions->move(5, 445);
    mass_actions->setAlignment(Qt::AlignLeft);
    mass_actions->setAlignment(Qt::AlignVCenter);
    mass_actions->setFixedSize(200, 25);

    auto clear_edges_button = new QPushButton("Clear Edges", this);
    clear_edges_button->setGeometry(5, 470, 92, 25);
    QPushButton::connect(clear_edges_button, &QPushButton::clicked, this, &controller::rem_all_edges);

    auto rem_all_nodes_button = new QPushButton("Clear Nodes", this);
    rem_all_nodes_button->setGeometry(102, 470, 93, 25);
    QPushButton::connect(rem_all_nodes_button, &QPushButton::clicked, this, &controller::rem_all_nodes); 

    auto show_degree = new QPushButton("Degrees", this);
    show_degree->setGeometry(5, 500, 92, 25);
    QPushButton::connect(show_degree, &QPushButton::clicked, this, [&](){
    	for(auto &[k, v] : nodes){
    		v->setText(QString::fromStdString(to_string(matrix.get_edges(k).size())));
    	}
    }); 

    auto show_index = new QPushButton("Indices", this);
    show_index->setGeometry(102, 500, 93, 25);
    QPushButton::connect(show_index, &QPushButton::clicked, this, [&](){
    	for(auto &[k, v] : nodes){
    		v->setText(QString::fromStdString(to_string(k)));
    	}
    });

    auto min_coloring = new QPushButton("Min Coloring", this);
    min_coloring->setGeometry(5, 530, 92, 25);
    QPushButton::connect(min_coloring, &QPushButton::clicked, this, [&](){
    	vector<tuple<const char*,const string>> colors = {
    		{"R", "#e50000"}, {"O", "#ff8d00"}, {"Y", "#ffee00"}, {"G", "#028121"}, {"B", "#004cff"}, {"P", "#770088"}};
    	auto coloring = matrix.minimal_coloring(6);
    	if(!coloring) return;
    	for(auto &[k, v] : *coloring) nodes[k]->color(get<1>(colors[v]));
    });

    k_value = new QLineEdit(this);
    k_value->move(102, 530);
    k_value->setFixedSize(22, 25);
    k_value->setPlaceholderText("K");

    auto k_coloring = new QPushButton("Color", this);
    k_coloring->setGeometry(129, 530, 66, 25); //102
    QPushButton::connect(k_coloring, &QPushButton::clicked, this, [&](){
    	vector<tuple<const char*,const string>> colors = {
    		{"R", "#e50000"}, {"O", "#ff8d00"}, {"Y", "#ffee00"}, {"G", "#028121"}, {"B", "#004cff"}, {"P", "#770088"}};
		uint32_t k = 3;
		try{ k = stol(k_value->text().toStdString());
    	} catch(...){ return; }
		if(k > 6) return;
		auto coloring = matrix.k_color(k);
		if(!coloring) return;
		for(auto &[k, v] : *coloring) nodes[k]->color(get<1>(colors[v]));
    });


    //TODO: Make a "graph search" class for these
    QLabel* graph_search = new QLabel("Graph Search", this);
    graph_search->move(5, 325);
    graph_search->setAlignment(Qt::AlignLeft);
    graph_search->setAlignment(Qt::AlignVCenter);
    graph_search->setFixedSize(200, 25);

    search_start = new QLineEdit(this);
    search_start->move(5, 350);
    search_start->setFixedSize(92, 25);
    search_start->setPlaceholderText("Start Node");

    search_end = new QLineEdit(this);
    search_end->move(102, 350);
    search_end->setFixedSize(93, 25);
    search_end->setPlaceholderText("End Node");

    auto bfs_button = new QPushButton("BFS", this);
    bfs_button->setGeometry(5, 380, 60, 40);
    QPushButton::connect(bfs_button, &QPushButton::clicked, this, [&](){
    	uint32_t start_node, end_node;
    	try{
	    	start_node=stol(search_start->text().toStdString());
	    	end_node=stol(search_end->text().toStdString());
    	} catch(...){ return; }
    	if(nodes.find(start_node) == nodes.end() || nodes.find(end_node) == nodes.end()) return;
    	auto to_highlight = matrix.BFS(start_node, end_node);
    	highlight_nodes(unordered_set<uint32_t>(to_highlight.begin(), to_highlight.end()));
    });

    auto dfs_button = new QPushButton("DFS", this);
    dfs_button->setGeometry(70, 380, 60, 40);
    QPushButton::connect(dfs_button, &QPushButton::clicked, this, [&](){
    	uint32_t start_node, end_node;
    	try{
	    	start_node=stol(search_start->text().toStdString());
	    	end_node=stol(search_end->text().toStdString());
    	} catch(...){ return; }
    	if(nodes.find(start_node) == nodes.end() || nodes.find(end_node) == nodes.end()) return;
    	auto to_highlight = matrix.DFS(start_node, end_node);
    	highlight_nodes(unordered_set<uint32_t>(to_highlight.begin(), to_highlight.end()));
    });

    auto ids_button = new QPushButton("IDS", this);
    ids_button->setGeometry(135, 380, 60, 40);
    QPushButton::connect(ids_button, &QPushButton::clicked, this, [&](){
    	uint32_t start_node, end_node;
    	try{
	    	start_node=stol(search_start->text().toStdString());
	    	end_node=stol(search_end->text().toStdString());
    	} catch(...){ return; }
    	if(nodes.find(start_node) == nodes.end() || nodes.find(end_node) == nodes.end()) return;
    	auto to_highlight = matrix.IDS(start_node, end_node);
    	highlight_nodes(unordered_set<uint32_t>(to_highlight.begin(), to_highlight.end()));
    });


    //Debug buttons
    auto test_button = new QPushButton("Debug: Test New Feature", this);
    test_button->setGeometry(0, 680, 200, 60);
    QPushButton::connect(test_button, &QPushButton::clicked, this, [&](){});

    auto debug_print_button = new QPushButton("Debug: Print Graph", this);
    debug_print_button->setGeometry(0, 740, 200, 60);
    QPushButton::connect(debug_print_button, &QPushButton::clicked, this, &controller::debug_print_graph);


    area = new graph_area(800, 800, this);
    area->move(200, 0);
}


void controller::right_click(uint32_t node_index){
	switch(mode){
		case REM_NODE: rem_node(node_index); break;
		case ADD_EDGE: 
			if(!currently_selected) break;
			new_edge(currently_selected->index, node_index); break;
		case REM_EDGE: 
			if(!currently_selected) break;
			rem_edge(matrix.get_edge(currently_selected->index, node_index)); break;
	}
}


void controller::new_node(){
	uint32_t node_index = matrix.new_node();
	nodes[node_index] = new node_label(QString::fromStdString(to_string(node_index)), area, node_index);
	sel_node(node_index);
}


void controller::rem_node(uint32_t node_index){
	node_label* node = nodes[node_index];
	if(currently_selected == node) currently_selected = NULL;
	for(auto edge : matrix.get_edges(node->index))
		rem_edge(edge);
	matrix.rem_node(node_index);
	nodes.erase(node_index);
	node->setParent(nullptr);
	node->deleteLater();
}


void controller::rem_all_nodes(){
	vector<uint32_t> node_indices;
	for(auto &[key, _] : nodes) node_indices.push_back(key);
	for(auto index : node_indices) rem_node(index);
}


void controller::sel_node(uint32_t node_index){
	if(currently_selected) currently_selected->deselect();
	currently_selected = nodes[node_index];
	currently_selected->select();
}


void controller::move_sel(int x, int y){
	if(!currently_selected) return;
	currently_selected->move(x-15, y-15); //TODO: Magic Number
	for(auto edge : matrix.get_edges(currently_selected->index)){
		edges[edge]->update();
	}
}


void controller::new_edge(uint32_t head_index, uint32_t tail_index){
	uint32_t edge_index = matrix.add_edge(head_index, tail_index, 1);
	auto edge_weights = matrix.get_weights(edge_index);
	if(edges.find(edge_index) == edges.end()) 
		edges[edge_index] = new edge_widget(head_index, tail_index, edge_weights, area);
	else edges[edge_index]->update_weights(edge_weights);
}


void controller::rem_edge(uint32_t edge_index){
	if(edge_index == 0) return;
	edge_widget* edge = edges[edge_index];
	matrix.rem_edge(edge->head_index, edge->tail_index);
	edges.erase(edge_index);
	edge->setParent(nullptr);
	edge->deleteLater();
}


void controller::rem_all_edges(){
	vector<uint32_t> edge_indices;
	for(auto &[key, _] : edges) edge_indices.push_back(key);
	for(auto index : edge_indices) rem_edge(index);
}


void controller::color_node(string color){
	if(currently_selected == nullptr) return;
	currently_selected->color(color);
}


void controller::set_node_label(string label){
	if(currently_selected == nullptr) return;
	currently_selected->setText(QString::fromStdString(label));
}


void controller::highlight_nodes(unordered_set<uint32_t> node_indices){
	for(auto &[k, v] : nodes){
		if(node_indices.find(k) != node_indices.end()) v->highlight();
		else if(v == currently_selected) v->select(); //Is it possible for this to be null?
		else v->deselect();
	}
}


void controller::debug_print_graph(){ matrix.print(); }