#pragma once
#include <QApplication>
#include <QPushButton>
#include <QWidget>
#include <QLabel>
#include <QMouseEvent>
#include <QPalette>
#include <QDebug>
#include <QPainter>
#include <QRadioButton>
#include <QPainterPath>
#include <QString>
#include <QLineEdit>

#include "graph.hpp"
using namespace std;


enum Mode { REM_NODE, ADD_EDGE, REM_EDGE };


class controller: public QWidget {
	Q_OBJECT
	class graph_area; friend class graph_area;
	class node_label; friend class node_label;
	class edge_widget; friend class edge_widget;
	class mode_switch; friend class mode_switch;	
	class selected_pane; friend class selected_pane;

	unordered_map<uint32_t,node_label*> nodes;
	unordered_map<uint32_t,edge_widget*> edges;
	selected_pane* selected;
	node_label* currently_selected = NULL; //TODO: Can this be contained in the selected_pane class?
	graph_area* area;
	adj_matrix matrix;
	Mode mode = REM_NODE;
	//TODO: Put these in their own QWidget class(es)
	QLineEdit* search_start;
	QLineEdit* search_end;
	QLineEdit* k_value;

	public:
	controller(QWidget* window);
	void right_click(uint32_t node_index);
	void new_node();
	void rem_node(uint32_t node_index);
	void rem_all_nodes();
	void sel_node(uint32_t node_index);
	void move_sel(int x, int y);
	void new_edge(uint32_t head_index, uint32_t tail_index);
	void rem_edge(uint32_t edge_index);
	void rem_all_edges();
	void color_node(string color);
	void set_node_label(string label);
	void highlight_nodes(unordered_set<uint32_t> node_indices);
	void debug_print_graph();
};


class controller::graph_area: public QWidget {
	Q_OBJECT
	public:
	controller* cont = NULL;

	graph_area(int x, int y, controller* parent);
	controller* get_controller();
	void mousePressEvent(QMouseEvent *event) override;
};


class controller::node_label: public QLabel {
	Q_OBJECT
	public:
	string current_color = "lightgrey";
	uint32_t index;

	node_label(const QString &text, graph_area* parent, uint32_t node);
	void style(string outline, string color);
	void color(string color);
	void select();
	void deselect();
	void highlight();
    void mousePressEvent(QMouseEvent *event) override;
};


class controller::edge_widget: public QWidget {
	Q_OBJECT
	public:
	uint32_t head_index;
	uint32_t tail_index;
	unordered_multiset<uint32_t> weights;

	edge_widget(uint32_t head, uint32_t tail, unordered_multiset<uint32_t> weight_set, graph_area* parent);
	void update_weights(unordered_multiset<uint32_t>);
	controller* get_controller();
	void draw_curve(QPainter* painter, int32_t distance);
	void draw_circle(QPainter* painter, uint32_t radius);
	void paintEvent(QPaintEvent* event) override;
};


class controller::mode_switch : public QWidget {
	Q_OBJECT
	public:
	QRadioButton* rem_node_radio;
    QRadioButton* add_edge_radio;
    QRadioButton* rem_edge_radio;

	mode_switch(controller* parent);
	controller* get_controller();
	void to_rem_node();
	void to_add_edge();
	void to_rem_edge();
};


class controller::selected_pane : public QWidget {
	Q_OBJECT
	public:
	node_label* selected;
	QLineEdit* color_text_box;
	QLineEdit* label_text_box;

	selected_pane(controller* parent);
	controller* get_controller();
};