#include <cmath>
#include "controller.hpp"


controller::edge_widget::edge_widget(uint32_t head, uint32_t tail, unordered_multiset<uint32_t> weight_set, graph_area* parent): QWidget(parent){
	head_index = head;
	tail_index = tail;
	weights = weight_set;
	resize(parent->width(), parent->height());
    lower();
	show();
	update();
}


void controller::edge_widget::draw_curve(QPainter* painter, int32_t distance){
    node_label* head = get_controller()->nodes[head_index];
    node_label* tail = get_controller()->nodes[tail_index];

	distance *= 2;
	double angle = -atan((double)(head->x()-tail->x()) / (head->y()-tail->y()));
    uint32_t mid_x = (head->x()+tail->x()) / 2;
    uint32_t mid_y = (head->y()+tail->y()) / 2;

    QPainterPath curve;
	curve.moveTo(head->x()+15, head->y()+15);
	curve.quadTo(mid_x+(distance*cos(angle))+15, mid_y+(distance*sin(angle))+15, tail->x()+15, tail->y()+15);
	painter->drawPath(curve);
}


void controller::edge_widget::draw_circle(QPainter* painter, uint32_t radius){
	node_label* head = get_controller()->nodes[head_index];
	double distance = sqrt(pow(385-head->x(), 2) + pow(385-head->y(), 2));
	double angle = atan((385.0-head->x())/(385.0-head->y()));
	double cent_x = 400 + ((distance + radius) * sin(angle));
	double cent_y = 400 + ((distance + radius) * cos(angle));
	if(head->y() <= 385){
		cent_x = 800-cent_x;
		cent_y = 800-cent_y;
	}
	painter->drawEllipse(cent_x-radius, cent_y-radius, 2*radius, 2*radius);
}


void controller::edge_widget::update_weights(unordered_multiset<uint32_t> new_weights){
	weights = new_weights;
	update();
}


void controller::edge_widget::paintEvent(QPaintEvent* event){
	QPainter painter (this);
	painter.setRenderHint(QPainter::Antialiasing, true);
    painter.setPen(QPen(Qt::black, 2));
	
	//FIXME: Sometimes adding a loop will remove the ability to add parallel edges
    if(head_index == tail_index){
    	for(uint32_t i = 0; i < weights.size(); i++)
    		draw_circle(&painter, 10*(i+2));
    } else {
		uint32_t distance = (1.0/(weights.size())) * 300;
		uint32_t start = (weights.size()%2) ? 0 : -(distance/2);
		for(uint32_t i = 0; i < weights.size(); i++){
			if(i%2) draw_curve(&painter, -(start+(((i+1)/2)*distance)));
			else draw_curve(&painter, (-start)+((i/2)*distance));
		}
    }

}


controller* controller::edge_widget::get_controller(){
	return qobject_cast<controller*>(parent()->parent());
}