#pragma once
#include <iostream>
#include <cstdint>
#include <vector>
#include <unordered_map>
#include <unordered_set>

using namespace std;


class Graph{
public:
	virtual vector<uint32_t> get_adj(uint32_t node) = 0;
	virtual bool is_adjacent(uint32_t head, uint32_t tail) = 0;
	virtual void print_node(uint32_t node) = 0;

	//Breadth First Search
	vector<uint32_t> BFS(uint32_t s, uint32_t e);
	//Depth First Search
	vector<uint32_t> DFS(uint32_t s, uint32_t e, uint32_t max_depth=UINT32_MAX);
	//Iterative Deepening Depth First Search
	vector<uint32_t> IDS(uint32_t s, uint32_t e);
};


//Directed/Undirected
//Weighted/Unweighted
// enum graph_type {  };
class adj_matrix : public Graph {
	private:
	uint32_t last_node = UINT32_MAX;
	unordered_map<uint32_t,unordered_map<uint32_t,uint32_t>> matrix;
	//Edge 0 represents no edge between the two nodes
	uint32_t last_edge = 0;
	unordered_map<uint32_t,unordered_multiset<uint32_t>> edge_weights = {{0, {0}}};

	public:
	adj_matrix(){}
	virtual vector<uint32_t> get_adj(uint32_t node);
	virtual bool is_adjacent(uint32_t head, uint32_t tail);
	virtual void print_node(uint32_t node);
	uint32_t new_node();
	void rem_node(uint32_t index);
	uint32_t add_edge(uint32_t head, uint32_t tail, uint32_t weight);
	void rem_edge(uint32_t head, uint32_t tail);
	// void new_weight(uint32_t edge, uint32_t weight);
	uint32_t get_edge(uint32_t head, uint32_t tail);
	unordered_set<uint32_t> get_edges(uint32_t node_index);
	unordered_multiset<uint32_t> get_weights(uint32_t edge_index);
	bool check_coloring(unordered_map<uint32_t,uint32_t> coloring);
	unordered_map<uint32_t,uint32_t>* k_color(uint32_t k);
	unordered_map<uint32_t,uint32_t>* minimal_coloring(uint32_t max);
	void print();
};